import os
import random
from unittest import TestCase


class EncoderDecoderTestCase(TestCase):
    def test_encoder(self):
        from raptorq import Encoder

        data = os.urandom(1024)
        encoder = Encoder.with_defaults(data, 512)
        packets = encoder.get_encoded_packets(42)

        self.assertIsInstance(packets, list)
        self.assertGreater(len(packets), 0)
        for packet in packets:
            self.assertIsInstance(packet, bytes)

    def test_decoder(self):
        from raptorq import Encoder, Decoder

        data = os.urandom(1024)
        encoder = Encoder.with_defaults(data, 512)
        packets = encoder.get_encoded_packets(42)

        random.shuffle(packets)

        decoded_data = None
        decoder = Decoder.with_defaults(len(data), 512)
        for packet in packets:
            decoded_data = decoder.decode(packet)
            if decoded_data is not None:
                break

        self.assertEqual(decoded_data, data)



import os
import random
from raptorq import Encoder, Decoder


def main():
    # Generate some random data to send
    data = os.urandom(10000)

    # Create the Encoder, with an MTU of 1400 (common for Ethernet)
    encoder = Encoder.with_defaults(data, 1400)

    # Perform the encoding, and serialize to bytes for transmission
    packets = encoder.get_encoded_packets(15)

    # Here we simulate losing 10 of the packets randomly. Normally, you would send them over
    # (potentially lossy) network here.
    random.shuffle(packets)
    # Erase 10 packets at random
    packets = packets[:-10]

    # The Decoder MUST be constructed with the configuration of the Encoder.
    # The configuration should be transmitted over a reliable channel
    decoder = Decoder.with_defaults(len(data), 1400)

    # Perform the decoding
    result = None
    for packet in packets:
        result = decoder.decode(packet)
        if result is not None:
            break

    # Check that even though some of the data was lost we are able to reconstruct the original message
    assert result == data


if __name__ == '__main__':
    main()
